@@ -23,7 +23,17 @@
 #ifndef BOTH_H
 #define BOTH_H
 
+/* provided by both.c */
+
+void *xmalloc(size_t s);
+void *xrealloc(void *p, size_t s);
+char *xstrsave(const char *s);
+
 int working_getc(FILE *file);
 size_t working_fread(void *ptr, size_t sz, FILE *file);
 
+/* used by both.c, so must be present */
+
+void syscallerror(const char *what) NONRETURNING;
+
 #endif
